from fastapi import APIRouter
from fastapi.exceptions import HTTPException
from mcapi.mc import IMC, AIOMC
from os import environ
from mcapi.models import AllInfo, BoolSchema, PlainString, PlainInt, TupleSchema
from functools import wraps
from fastapi_cache.decorator import cache

# Minecraft server connection vars
MC_HOST = environ["MC_HOST"]
MC_PORT = int(environ["MC_PORT"])

# Routers settings``
default = APIRouter(tags=["APIv1", "Server"])
players = APIRouter(prefix="/players", tags=["APIv1", "Players"])

# Implement Minecraft socket api
mc: IMC
mc = AIOMC(MC_HOST, MC_PORT)


# Connection error handler
def cre(prop):
    @wraps(prop)
    async def wrapper():
        try:
            return await prop()
        except ConnectionRefusedError:
            raise HTTPException(
                status_code=503, detail="Refused to connect to the Minecraft server"
            )
        except ConnectionResetError:
            raise HTTPException(
                status_code=503, detail="Connection reset by Minecraft server"
            )

    return wrapper


@default.get("/name")
@cache()
@cre
async def server_name() -> PlainString:
    """
    Returns server core name
    """
    return await mc.name


@default.get("/status")
@cache(expire=60)
async def server_status() -> BoolSchema:
    """
    Returns server online status
    """
    return await mc.status


@default.get("/info")
@cache(expire=60)
@cre
async def server_info() -> AllInfo:
    """
    Returns Server info
    """
    return await mc.all


@players.get("/count")
@cache(expire=120)
@cre
async def players_count() -> PlainInt:
    """
    Returns online players count
    """
    return await mc.players_count


@players.get("/max")
@cache()
@cre
async def players_max() -> PlainInt:
    """
    Returns max players count
    """
    return await mc.max_players


@players.get("/list")
@cache(expire=120)
@cre
async def players_list() -> TupleSchema:
    """
    Returns online players list
    """
    return await mc.players


@default.get("/health")
async def healthcheck():
    status = await mc.status
    if status.data:
        return 0
    return 1
